@@ -1,12 +1,12 @@
 import asyncio
-from typing import cast, Awaitable
+from typing import cast, Any, Awaitable, Optional
 
 from pytest import raises, mark  # type: ignore
 
 from graphql.error import GraphQLError
-from graphql.execution import execute
-from graphql.language import parse, OperationDefinitionNode, FieldNode
-from graphql.pyutils import inspect
+from graphql.execution import execute, execute_sync
+from graphql.language import parse, FieldNode, OperationDefinitionNode
+from graphql.pyutils import inspect, Undefined
 from graphql.type import (
     GraphQLArgument,
     GraphQLBoolean,
@@ -18,13 +18,14 @@ from graphql.type import (
     GraphQLObjectType,
     GraphQLResolveInfo,
     GraphQLSchema,
+    GraphQLScalarType,
     GraphQLString,
+    GraphQLUnionType,
     ResponsePath,
 )
 
 
 def describe_execute_handles_basic_execution_tasks():
-
     # noinspection PyTypeChecker
     def throws_if_no_document_is_provided():
         schema = GraphQLSchema(
@@ -32,19 +33,52 @@ def describe_execute_handles_basic_execution_tasks():
         )
 
         with raises(TypeError) as exc_info:
-            assert execute(schema=schema, document=None)  # type: ignore
+            assert execute_sync(schema=schema, document=None)  # type: ignore
 
-        assert str(exc_info.value) == "Must provide document"
+        assert str(exc_info.value) == "Must provide document."
 
     # noinspection PyTypeChecker
     def throws_if_no_schema_is_provided():
         document = parse("{ field }")
 
         with raises(TypeError) as exc_info:
-            assert execute(schema=None, document=document)  # type: ignore
+            assert execute_sync(schema=None, document=document)  # type: ignore
 
         assert str(exc_info.value) == "Expected None to be a GraphQL schema."
 
+    def throws_on_invalid_variables():
+        schema = GraphQLSchema(
+            GraphQLObjectType(
+                "Type",
+                {
+                    "fieldA": GraphQLField(
+                        GraphQLString, args={"argA": GraphQLArgument(GraphQLInt)}
+                    )
+                },
+            )
+        )
+        document = parse(
+            """
+            query ($a: Int) {
+              fieldA(argA: $a)
+            }
+            """
+        )
+        variable_values = "{'a': 1}"
+
+        with raises(TypeError) as exc_info:
+            assert execute_sync(
+                schema=schema,
+                document=document,
+                variable_values=variable_values,  # type: ignore
+            )
+
+        assert str(exc_info.value) == (
+            "Variable values must be provided as a dictionary"
+            " with variable names as keys. Perhaps look to see"
+            " if an unparsed JSON string was provided."
+        )
+
     def accepts_positional_arguments():
         schema = GraphQLSchema(
             GraphQLObjectType(
@@ -53,13 +87,12 @@ def describe_execute_handles_basic_execution_tasks():
             )
         )
 
-        result = execute(schema, parse("{ a }"), "rootValue")
+        result = execute_sync(schema, parse("{ a }"), "rootValue")
 
         assert result == ({"a": "rootValue"}, None)
 
     @mark.asyncio
     async def executes_arbitrary_code():
-
         # noinspection PyMethodMayBeStatic,PyMethodMayBeStatic
         class Data:
             def a(self, _info):
@@ -80,7 +113,7 @@ def describe_execute_handles_basic_execution_tasks():
             f = "Fish"
 
             # Called only by DataType::pic static resolver
-            def pic(self, _info, size=50):
+            def pic(self, _info, size: int):
                 return f"Pic of size: {size}"
 
             def deep(self, _info):
@@ -228,7 +261,7 @@ def describe_execute_handles_basic_execution_tasks():
             """
         )
 
-        result = execute(schema, ast)
+        result = execute_sync(schema, ast)
         assert result == (
             {
                 "a": "Apple",
@@ -258,26 +291,80 @@ def describe_execute_handles_basic_execution_tasks():
         document = parse("query ($var: String) { result: test }")
         root_value = {"root": "val"}
         variable_values = {"var": "abc"}
-        execute(schema, document, root_value, variable_values=variable_values)
+        execute_sync(schema, document, root_value, variable_values=variable_values)
 
         assert len(resolved_infos) == 1
         operation = cast(OperationDefinitionNode, document.definitions[0])
         assert operation and operation.kind == "operation_definition"
         field = cast(FieldNode, operation.selection_set.selections[0])
+
         assert resolved_infos[0] == GraphQLResolveInfo(
             field_name="test",
             field_nodes=[field],
             return_type=GraphQLString,
             parent_type=cast(GraphQLObjectType, schema.query_type),
-            path=ResponsePath(None, "result"),
+            path=ResponsePath(None, "result", "Test"),
             schema=schema,
             fragments={},
             root_value=root_value,
             operation=operation,
             variable_values=variable_values,
             context=None,
+            is_awaitable=resolved_infos[0].is_awaitable,
         )
 
+    def it_populates_path_correctly_with_complex_types():
+        path: Optional[ResponsePath] = None
+
+        def resolve(_val, info):
+            nonlocal path
+            path = info.path
+
+        def resolve_type(_val, _info, _type):
+            return "SomeObject"
+
+        some_object = GraphQLObjectType(
+            "SomeObject", {"test": GraphQLField(GraphQLString, resolve=resolve)}
+        )
+        some_union = GraphQLUnionType(
+            "SomeUnion", [some_object], resolve_type=resolve_type
+        )
+        test_type = GraphQLObjectType(
+            "SomeQuery",
+            {
+                "test": GraphQLField(
+                    GraphQLNonNull(GraphQLList(GraphQLNonNull(some_union)))
+                )
+            },
+        )
+        schema = GraphQLSchema(test_type)
+        root_value: Any = {"test": [{}]}
+        document = parse(
+            """
+            query {
+              l1: test {
+                ... on SomeObject {
+                  l2: test
+                }
+              }
+            }
+            """
+        )
+
+        execute_sync(schema, document, root_value)
+
+        assert path is not None
+        prev, key, typename = path
+        assert key == "l2"
+        assert typename == "SomeObject"
+        prev, key, typename = prev
+        assert key == 0
+        assert typename is None
+        prev, key, typename = prev
+        assert key == "l1"
+        assert typename == "SomeQuery"
+        assert prev is None
+
     def threads_root_value_context_correctly():
         resolved_values = []
 
@@ -295,7 +382,7 @@ def describe_execute_handles_basic_execution_tasks():
 
         document = parse("query Example { a }")
         root_value = Data()
-        execute(schema, document, root_value)
+        execute_sync(schema, document, root_value)
 
         assert len(resolved_values) == 1
         assert resolved_values[0] is root_value
@@ -330,7 +417,7 @@ def describe_execute_handles_basic_execution_tasks():
             """
         )
 
-        execute(schema, document)
+        execute_sync(schema, document)
 
         assert len(resolved_args) == 1
         assert resolved_args[0] == {"numArg": 123, "stringArg": "foo"}
@@ -399,7 +486,7 @@ def describe_execute_handles_basic_execution_tasks():
                 return "async ok"
 
             async def asyncError(self, _info):
-                raise Exception("Error getting asyncError")
+                raise GraphQLError("Error getting asyncError")
 
             async def asyncRawError(self, _info):
                 raise Exception("Error getting asyncRawError")
@@ -518,7 +605,7 @@ def describe_execute_handles_basic_execution_tasks():
             """
         )
 
-        assert execute(schema, document) == (
+        assert execute_sync(schema, document) == (
             {"nullableA": {"aliasedA": None}},
             [
                 {
@@ -539,7 +626,7 @@ def describe_execute_handles_basic_execution_tasks():
         class Data:
             a = "b"
 
-        result = execute(schema, document, Data())
+        result = execute_sync(schema, document, Data())
         assert result == ({"a": "b"}, None)
 
     def uses_the_only_operation_if_no_operation_name_is_provided():
@@ -552,7 +639,7 @@ def describe_execute_handles_basic_execution_tasks():
         class Data:
             a = "b"
 
-        result = execute(schema, document, Data())
+        result = execute_sync(schema, document, Data())
         assert result == ({"a": "b"}, None)
 
     def uses_the_named_operation_if_operation_name_is_provided():
@@ -570,7 +657,7 @@ def describe_execute_handles_basic_execution_tasks():
         class Data:
             a = "b"
 
-        result = execute(schema, document, Data(), operation_name="OtherExample")
+        result = execute_sync(schema, document, Data(), operation_name="OtherExample")
         assert result == ({"second": "b"}, None)
 
     def provides_error_if_no_operation_is_provided():
@@ -583,7 +670,7 @@ def describe_execute_handles_basic_execution_tasks():
         class Data:
             a = "b"
 
-        result = execute(schema, document, Data())
+        result = execute_sync(schema, document, Data())
         assert result == (None, [{"message": "Must provide an operation."}])
 
     def errors_if_no_operation_name_is_provided_with_multiple_operations():
@@ -598,7 +685,7 @@ def describe_execute_handles_basic_execution_tasks():
             """
         )
 
-        result = execute(schema, document)
+        result = execute_sync(schema, document)
         assert result == (
             None,
             [
@@ -621,12 +708,25 @@ def describe_execute_handles_basic_execution_tasks():
             """
         )
 
-        result = execute(schema, document, operation_name="UnknownExample")
+        result = execute_sync(schema, document, operation_name="UnknownExample")
         assert result == (
             None,
             [{"message": "Unknown operation named 'UnknownExample'."}],
         )
 
+    def errors_if_empty_string_is_provided_as_operation_name():
+        schema = GraphQLSchema(
+            GraphQLObjectType("Type", {"a": GraphQLField(GraphQLString)})
+        )
+
+        document = parse("{ a }")
+
+        result = execute_sync(schema, document, operation_name="")
+        assert result == (
+            None,
+            [{"message": "Unknown operation named ''."}],
+        )
+
     def uses_the_query_schema_for_queries():
         schema = GraphQLSchema(
             GraphQLObjectType("Q", {"a": GraphQLField(GraphQLString)}),
@@ -646,7 +746,7 @@ def describe_execute_handles_basic_execution_tasks():
             a = "b"
             c = "d"
 
-        result = execute(schema, document, Data(), operation_name="Q")
+        result = execute_sync(schema, document, Data(), operation_name="Q")
         assert result == ({"a": "b"}, None)
 
     def uses_the_mutation_schema_for_mutations():
@@ -666,7 +766,7 @@ def describe_execute_handles_basic_execution_tasks():
             a = "b"
             c = "d"
 
-        result = execute(schema, document, Data(), operation_name="M")
+        result = execute_sync(schema, document, Data(), operation_name="M")
         assert result == ({"c": "d"}, None)
 
     def uses_the_subscription_schema_for_subscriptions():
@@ -686,7 +786,7 @@ def describe_execute_handles_basic_execution_tasks():
             a = "b"
             c = "d"
 
-        result = execute(schema, document, Data(), operation_name="S")
+        result = execute_sync(schema, document, Data(), operation_name="S")
         assert result == ({"a": "b"}, None)
 
     @mark.asyncio
@@ -752,10 +852,21 @@ def describe_execute_handles_basic_execution_tasks():
         class Data:
             a = "b"
 
-        result = execute(schema, document, Data(), operation_name="Q")
+        result = execute_sync(schema, document, Data(), operation_name="Q")
 
         assert result == ({"a": "b"}, None)
 
+    def ignores_missing_sub_selections_on_fields():
+        some_type = GraphQLObjectType("SomeType", {"b": GraphQLField(GraphQLString)})
+        schema = GraphQLSchema(
+            GraphQLObjectType("Query", {"a": GraphQLField(some_type)})
+        )
+        document = parse("{ a }")
+        root_value = {"a": {"b": "c"}}
+
+        result = execute_sync(schema, document, root_value)
+        assert result == ({"a": {}}, None)
+
     def does_not_include_illegal_fields_in_output():
         schema = GraphQLSchema(
             GraphQLObjectType("Q", {"a": GraphQLField(GraphQLString)})
@@ -763,7 +874,7 @@ def describe_execute_handles_basic_execution_tasks():
 
         document = parse("{ thisIsIllegalDoNotIncludeMe }")
 
-        result = execute(schema, document)
+        result = execute_sync(schema, document)
 
         assert result == ({}, None)
 
@@ -789,12 +900,13 @@ def describe_execute_handles_basic_execution_tasks():
 
         document = parse("{ field(a: true, c: false, e: 0) }")
 
-        assert execute(schema, document) == (
+        assert execute_sync(schema, document) == (
             {"field": "{'a': True, 'c': False, 'e': 0}"},
             None,
         )
 
-    def fails_when_an_is_type_of_check_is_not_met():
+    @mark.asyncio
+    async def fails_when_is_type_of_check_is_not_met():
         class Special:
             value: str
 
@@ -807,10 +919,20 @@ def describe_execute_handles_basic_execution_tasks():
             def __init__(self, value):
                 self.value = value
 
+        def is_type_of_special(obj, _info):
+            is_special = isinstance(obj, Special)
+            if not _info.context["async"]:
+                return is_special
+
+            async def async_is_special():
+                return is_special
+
+            return async_is_special()
+
         SpecialType = GraphQLObjectType(
             "SpecialType",
             {"value": GraphQLField(GraphQLString)},
-            is_type_of=lambda obj, _info: isinstance(obj, Special),
+            is_type_of=is_type_of_special,
         )
 
         schema = GraphQLSchema(
@@ -822,7 +944,8 @@ def describe_execute_handles_basic_execution_tasks():
         document = parse("{ specials { value } }")
         root_value = {"specials": [Special("foo"), NotSpecial("bar")]}
 
-        result = execute(schema, document, root_value)
+        result = execute_sync(schema, document, root_value, {"async": False})
+        assert not isinstance(result, Awaitable)
         assert result == (
             {"specials": [{"value": "foo"}, None]},
             [
@@ -835,6 +958,39 @@ def describe_execute_handles_basic_execution_tasks():
             ],
         )
 
+        async_result = execute(schema, document, root_value, {"async": True})
+        assert isinstance(async_result, Awaitable)
+        awaited_result = await async_result
+        assert awaited_result == result
+
+    def fails_when_serialize_of_custom_scalar_does_not_return_a_value():
+        custom_scalar = GraphQLScalarType(
+            "CustomScalar", serialize=lambda _value: Undefined  # returns nothing
+        )
+        schema = GraphQLSchema(
+            GraphQLObjectType(
+                "Query",
+                {
+                    "customScalar": GraphQLField(
+                        custom_scalar, resolve=lambda *_args: "CUSTOM_VALUE"
+                    )
+                },
+            )
+        )
+
+        result = execute_sync(schema, parse("{ customScalar }"))
+        assert result == (
+            {"customScalar": None},
+            [
+                {
+                    "message": "Expected a value of type 'CustomScalar'"
+                    " but received: 'CUSTOM_VALUE'",
+                    "locations": [(1, 3)],
+                    "path": ["customScalar"],
+                }
+            ],
+        )
+
     def executes_ignoring_invalid_non_executable_definitions():
         schema = GraphQLSchema(
             GraphQLObjectType("Query", {"foo": GraphQLField(GraphQLString)})
@@ -848,7 +1004,7 @@ def describe_execute_handles_basic_execution_tasks():
             """
         )
 
-        result = execute(schema, document)
+        result = execute_sync(schema, document)
         assert result == ({"foo": None}, None)
 
     def uses_a_custom_field_resolver():
@@ -857,11 +1013,11 @@ def describe_execute_handles_basic_execution_tasks():
         )
         document = parse("{ foo }")
 
-        def field_resolver(source_, info):
+        def field_resolver(_source, info):
             # For the purposes of test, just return the name of the field!
             return info.field_name
 
-        result = execute(schema, document, field_resolver=field_resolver)
+        result = execute_sync(schema, document, field_resolver=field_resolver)
         assert result == ({"foo": "foo"}, None)
 
     def uses_a_custom_type_resolver():
@@ -889,7 +1045,7 @@ def describe_execute_handles_basic_execution_tasks():
             return "FooObject"
 
         root_value = {"foo": {"bar": "bar"}}
-        result = execute(schema, document, root_value, type_resolver=type_resolver)
+        result = execute_sync(schema, document, root_value, type_resolver=type_resolver)
 
         assert result == ({"foo": {"bar": "bar"}}, None)
         assert possible_types == [foo_object]
