@@ -5,7 +5,9 @@ from pytest import raises  # type: ignore
 from graphql.error import GraphQLSyntaxError
 from graphql.language import Lexer, Source, SourceLocation, Token, TokenKind
 from graphql.language.lexer import is_punctuator_token_kind
-from graphql.pyutils import dedent, inspect
+from graphql.pyutils import inspect
+
+from ..utils import dedent
 
 
 def lex_one(s: str) -> Token:
@@ -354,13 +356,17 @@ def describe_lexer():
             "1_1234", "Invalid number, expected digit but got: '_'.", (1, 2)
         )
         assert_syntax_error(
-            "1ß", "Cannot parse the unexpected character 'ß'.", (1, 2),
+            "1ß",
+            "Cannot parse the unexpected character 'ß'.",
+            (1, 2),
         )
         assert_syntax_error(
             "1.23f", "Invalid number, expected digit but got: 'f'.", (1, 5)
         )
         assert_syntax_error(
-            "12ß", "Cannot parse the unexpected character 'ß'.", (1, 3),
+            "12ß",
+            "Cannot parse the unexpected character 'ß'.",
+            (1, 3),
         )
 
     # noinspection PyArgumentEqualDefault
