@@ -1,7 +1,6 @@
 from functools import partial
 
 from graphql.validation import UniqueVariableNamesRule
-from graphql.validation.rules.unique_variable_names import duplicate_variable_message
 
 from .harness import assert_validation_errors
 
@@ -10,13 +9,6 @@ assert_errors = partial(assert_validation_errors, UniqueVariableNamesRule)
 assert_valid = partial(assert_errors, errors=[])
 
 
-def duplicate_variable(name, l1, c1, l2, c2):
-    return {
-        "message": duplicate_variable_message(name),
-        "locations": [(l1, c1), (l2, c2)],
-    }
-
-
 def describe_validate_unique_variable_names():
     def unique_variable_names():
         assert_valid(
@@ -34,9 +26,21 @@ def describe_validate_unique_variable_names():
             query C($x: Int, $x: Int) { __typename }
             """,
             [
-                duplicate_variable("x", 2, 22, 2, 31),
-                duplicate_variable("x", 2, 22, 2, 40),
-                duplicate_variable("x", 3, 22, 3, 34),
-                duplicate_variable("x", 4, 22, 4, 31),
+                {
+                    "message": "There can be only one variable named '$x'.",
+                    "locations": [(2, 22), (2, 31)],
+                },
+                {
+                    "message": "There can be only one variable named '$x'.",
+                    "locations": [(2, 22), (2, 40)],
+                },
+                {
+                    "message": "There can be only one variable named '$x'.",
+                    "locations": [(3, 22), (3, 34)],
+                },
+                {
+                    "message": "There can be only one variable named '$x'.",
+                    "locations": [(4, 22), (4, 31)],
+                },
             ],
         )
